#ifndef __LIBHFZ_INCLUDED__
#define __LIBHFZ_INCLUDED__

#pragma once

#include "zlib\include\zlib.h"

#define FORMAT_HF2		0
#define FORMAT_HF2_GZ	1

#define ERROR_OK				0
#define ERROR_OPEN_FAILED		-1
#define ERROR_WRITE_FAILED		-2
#define ERROR_READ_FAILED		-3
#define ERROR_ALLOC_FAILED		-4
#define ERROR_INVALID_HANDLE	-5
#define ERROR_INVALID_FORMAT	-6
#define ERROR_INVALID_PRECIS	-7
#define ERROR_INVALID_MAP_SIZE	-8
#define ERROR_INVALID_TILE_SIZE	-9
#define ERROR_WRITE_HEADER_FAILED	-10
#define ERROR_WRITE_EXTHEAD_FAILED	-11

// HFZ v1.0 header
typedef struct {
	unsigned short FileVersionNo;
	unsigned long nx;
	unsigned long ny;
	unsigned short TileSize;
	float HorizScale;
	float Precis;
	unsigned long ExtHeaderLength;
	void* pExtHeaderData;
} hfzHeader;

typedef struct {
	char BlockType[4];
	char BlockName[16];
	unsigned long BlockLength;
	void* pBlockData;
} hfzBlockHeader;

typedef struct {
	long Format;
	struct _iobuf *raw;
	gzFile gz;
} hfzFile;

// high-level
long hfzSave(const char* lpFileName, long Format, float* pData, long nx, long ny, int TileSize, float Precis, float HorizScale, char* lpExtHeader, long ExtHeaderLength);
long hfzLoad(const char* lpFileName, hfzHeader& fh, float * pData); // user must allocate map mem
long hfzLoadEx(const char* lpFileName, hfzHeader& fh, float ** h_pData); // allocates map mem (user must free)

// medium-level
long hfzReadHeader(hfzFile* fs, hfzHeader& fh);
long hfzReadHeader2(const char* lpFileName, hfzHeader& fh);
long hfzReadTile(hfzFile* fs, hfzHeader& fh, long TileX, long TileY, float* pMapData);

long hfzWriteHeader(hfzFile* fs, hfzHeader& fh);
long hfzWriteTile(hfzFile* fs, hfzHeader& fh, long TileX, long TileY, float* pMapData);

// low-level
hfzFile* hfzOpen(const char* lpFileName, long Format, const char* lpOptions);
void hfzClose(hfzFile* fs);
long hfzWrite(hfzFile* fs, void* pData, int len);
long hfzRead(hfzFile* fs, void* pData, int len);

// helper
const char* hfzGetErrorStr(long ErrorCode);

void hfzResetHeader(hfzHeader &fh); // clears mem in ext header buf and restores defaults

#endif